import com.sun.javadoc.*;
import java.io.*;

/** Diese Klasse erzeugt eine XML Ausgabedatei in der alle
 * mit dem Javadoc Tag @todo ausgezeichneten Texte
 * aufgef&uuml;hrt sind.
 * @author Christoph Krger
 * @version 0.9
 * @todo Einbau der Kommandozeilenparameterbergabe
 * @todo Verarbeitung von HMTL Entities ermglichen
 */
public class ListTodos 
{
    /** Standardmethode f&uuml;r Doclets. Schreibt die XML-Datei
     * ins lokale Verzeichnis
     * @param root Das Root-Document f&uuml;r die dokumentierten
     * Klassen
     * @return wahr, wenn alles geklappt hat
     * @todo Verwenden eines Kommandozeilenparameters fr die Ausgabedatei
     * @todo Verwenden eines Kommandozeilenparameters fr das Stylesheet
     */    
    public static boolean start(RootDoc root)
    {
        String tagName = "todo";

        StringBuffer sbuff = new StringBuffer("<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n<?xml-stylesheet type=\"text/xsl\" href=\"todo.xsl\"?>\n<control>");
        sbuff.append(writeContents(root.classes(), tagName));
        sbuff.append("</control>");
        // Schreiben der XML-Datei auf die Platte
        try
        {
            FileOutputStream fos = new FileOutputStream("todo.xml");
            PrintWriter pw = new PrintWriter(fos,true);
            pw.print(sbuff.toString());
            pw.close();
            fos.close();
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }
        return true;
    }

    /** Interne Methode, die das XML durch einfache String-Operationen
     * erzeugt
     * @param classes Array der Klassen, die kommentiert sind
     * @param tagName Name des Tags, der die Todos einleitet
     * @return String mit Todos in XML-Auszeichnung
     * @todo Ausgabe des Packagenamens
     */    
    private static String writeContents(ClassDoc[] classes, String tagName)
    {
        StringBuffer sbuff = new StringBuffer();
        for (int i=0; i < classes.length; i++)
        {
            // Klassennamen immer hinschreiben
            sbuff.append("<class name=\"" + classes[i].name() + "\">\n");
            // Todo Tags hier finden
            Tag[] tags = classes[i].tags(tagName);
            // ... und ausgeben
            if (tags.length > 0)
            {
                    for (int k=0; k < tags.length; k++)
                    {
                            sbuff.append("<todo>" + tags[k].text() + "</todo>\n");
                    }
            }

            // jetzt durch die Methoden gehen
            MethodDoc[] methods = classes[i].methods();
            for (int j=0; j < methods.length; j++)
            {
                // Methodennamen ausgeben
                sbuff.append("<method name=\"" + methods[j].name() + "\">\n");
                tags = methods[j].tags(tagName);
                if (tags.length > 0)
                {
                    for (int k=0; k < tags.length; k++)
                    {
                        sbuff.append("<todo>" + tags[k].text() + "</todo>\n");
                    }
                }
                sbuff.append("</method>\n");
            }
            sbuff.append("</class>\n");
        }
        return sbuff.toString();
    }
}

